@@ -20,6 +20,7 @@ import argparse
 import os
 import socket
 import sys
+import xml
 import xmlrpc.client
 from urllib.parse import urlparse
 
@@ -42,6 +43,7 @@ from .commands import (
     results,
     system,
     tags,
+    tokens,
     users,
     utils,
     workers,
@@ -107,6 +109,10 @@ class RequestsTransport(xmlrpc.client.Transport):
                 raise xmlrpc.client.ProtocolError(
                     url, response.status_code, str(e), response.headers
                 )
+        except xml.parsers.expat.ExpatError as e:
+            raise xmlrpc.client.ProtocolError(
+                url, 500, f"Invalid response format, check your endpoint URL: {e}", ""
+            )
 
     def parse_response(self, response):
         """
@@ -195,6 +201,7 @@ def main():
         "results": results,
         "system": system,
         "tags": tags,
+        "tokens": tokens,
         "users": users,
         "utils": utils,
         "workers": workers,
@@ -237,9 +244,16 @@ def main():
             token = config.get("token")
             if username is not None and token is not None:
                 p = urlparse(config["uri"])
-                uri = f"{p.scheme}://{username}:{token}@{p.netloc}{p.path}"
+                path = "" if p.path == "/" else p.path
+                uri = f"{p.scheme}://{username}:{token}@{p.netloc}{path}"
             else:
-                uri = config["uri"]
+                p = urlparse(config["uri"])
+                path = "" if p.path == "/" else p.path
+                uri = f"{p.scheme}://{p.netloc}{path}"
+        else:
+            p = urlparse(uri)
+            path = "" if p.path == "/" else p.path
+            uri = f"{p.scheme}://{p.netloc}{path}"
 
         try:
             # Create the Transport object
