@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # vim: set ts=4
 
 # Copyright 2017 Rémi Duraffort
@@ -98,26 +97,55 @@ def configure_parser(parser, version):
         help="print as yaml",
     )
 
-    if version >= (2018, 4):
-        # "heath-check"
-        dt_hc = sub.add_parser("health-check", help="device-type health-check")
-        dt_sub = dt_hc.add_subparsers(dest="sub_sub_sub_command", help="Sub commands")
-        dt_sub.required = True
-        if version >= (2022, 4):
-            dt_delete = dt_sub.add_parser(
-                "delete", help="delete the device-type health-check"
-            )
-            dt_delete.add_argument("name", help="name of the device-type")
-
-        dt_get = dt_sub.add_parser("get", help="get the device-type health-check")
-        dt_get.add_argument("name", help="name of the device-type")
-
-        dt_set = dt_sub.add_parser("set", help="set the device-type health-check")
-        dt_set.add_argument("name", help="name of the device-type")
-        dt_set.add_argument(
-            "definition", type=argparse.FileType("r"), help="health-check definition"
+    # "devices"
+    if version >= (2025, 10):
+        dt_devices = sub.add_parser("devices", help="list devices for the device-type")
+        dt_devices.add_argument("name", help="Device-type")
+        dt_devices.add_argument(
+            "--health",
+            type=str,
+            default=None,
+            choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE"],
+            help="filter devices by health",
         )
 
+        out_format = dt_devices.add_mutually_exclusive_group()
+        out_format.add_argument(
+            "--json",
+            dest="output_format",
+            default=None,
+            action="store_const",
+            const="json",
+            help="print as json",
+        )
+        out_format.add_argument(
+            "--yaml",
+            dest="output_format",
+            default=None,
+            action="store_const",
+            const="yaml",
+            help="print as yaml",
+        )
+
+    # "heath-check"
+    dt_hc = sub.add_parser("health-check", help="device-type health-check")
+    dt_sub = dt_hc.add_subparsers(dest="sub_sub_sub_command", help="Sub commands")
+    dt_sub.required = True
+    if version >= (2022, 4):
+        dt_delete = dt_sub.add_parser(
+            "delete", help="delete the device-type health-check"
+        )
+        dt_delete.add_argument("name", help="name of the device-type")
+
+    dt_get = dt_sub.add_parser("get", help="get the device-type health-check")
+    dt_get.add_argument("name", help="name of the device-type")
+
+    dt_set = dt_sub.add_parser("set", help="set the device-type health-check")
+    dt_set.add_argument("name", help="name of the device-type")
+    dt_set.add_argument(
+        "definition", type=argparse.FileType("r"), help="health-check definition"
+    )
+
     # "list"
     dt_list = sub.add_parser("list", help="list available device-types")
     dt_list.add_argument(
@@ -146,6 +174,44 @@ def configure_parser(parser, version):
         help="print as yaml",
     )
 
+    # "perms"
+    if version >= (2023, 3):
+        dt_perms = sub.add_parser("perms", help="permissions")
+        perms_sub = dt_perms.add_subparsers(
+            dest="sub_sub_sub_command", help="Sub commands"
+        )
+        perms_sub.required = True
+
+        perms_add = perms_sub.add_parser("add", help="add permissions")
+        perms_add.add_argument("name", help="name of the device-type")
+        perms_add.add_argument("group", help="group")
+        perms_add.add_argument("permission", help="permission")
+
+        perms_delete = perms_sub.add_parser("delete", help="delete permissions")
+        perms_delete.add_argument("name", help="name of the device-type")
+        perms_delete.add_argument("group", help="group")
+        perms_delete.add_argument("permission", help="permission")
+
+        perms_list = perms_sub.add_parser("list", help="list permissions")
+        perms_list.add_argument("name", help="name of the device-type")
+        out_format = perms_list.add_mutually_exclusive_group()
+        out_format.add_argument(
+            "--json",
+            dest="output_format",
+            default=None,
+            action="store_const",
+            const="json",
+            help="print as json",
+        )
+        out_format.add_argument(
+            "--yaml",
+            dest="output_format",
+            default=None,
+            action="store_const",
+            const="yaml",
+            help="print as yaml",
+        )
+
     # "show"
     dt_show = sub.add_parser("show", help="show device-type details")
     dt_show.add_argument("name", help="name of the device-type")
@@ -297,6 +363,21 @@ def handle_hc(proxy, options, _):
     return 0
 
 
+def handle_devices(proxy, options, _):
+    devices = proxy.scheduler.device_types.devices(options.name, options.health)
+
+    if options.output_format == "json":
+        print(json.dumps(devices))
+    elif options.output_format == "yaml":
+        safe_yaml.dump(devices, sys.stdout)
+    else:
+        print("Devices:")
+        for d in devices:
+            print(f"* {d['hostname']} : {d['state']},{d['health']}")
+
+    return 0
+
+
 def handle_list(proxy, options, _):
     device_types = proxy.scheduler.device_types.list(options.show_all)
 
@@ -307,7 +388,29 @@ def handle_list(proxy, options, _):
     else:
         print("Device-Types:")
         for dt in device_types:
-            print("* %s (%s)" % (dt["name"], dt["devices"]))
+            print(f"* {dt['name']} ({dt['devices']})")
+    return 0
+
+
+def handle_perms(proxy, options, _):
+    if options.sub_sub_sub_command == "add":
+        proxy.scheduler.device_types.perms_add(
+            options.name, options.group, options.permission
+        )
+    elif options.sub_sub_sub_command == "delete":
+        proxy.scheduler.device_types.perms_delete(
+            options.name, options.group, options.permission
+        )
+    elif options.sub_sub_sub_command == "list":
+        perms = proxy.scheduler.device_types.perms_list(options.name)
+        if options.output_format == "json":
+            print(json.dumps(perms))
+        elif options.output_format == "yaml":
+            safe_yaml.dump(perms, sys.stdout)
+        else:
+            print("Permissions:")
+            for perm in perms:
+                print("* %s %s" % (perm["group"], perm["name"]))
     return 0
 
 
@@ -359,8 +462,10 @@ def handle(proxy, options, config):
     handlers = {
         "add": handle_add,
         "aliases": handle_aliases,
+        "devices": handle_devices,
         "health-check": handle_hc,
         "list": handle_list,
+        "perms": handle_perms,
         "show": handle_show,
         "template": handle_template,
         "update": handle_update,
