@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # vim: set ts=4
 
 # Copyright 2017 Rémi Duraffort
@@ -19,8 +18,10 @@
 
 import argparse
 import json
-import time
 import sys
+import time
+
+import jinja2
 
 from lavacli.utils import safe_yaml
 
@@ -40,26 +41,12 @@ def configure_parser(parser, version):
     owner.add_argument("--user", default=None, help="device owner")
     owner.add_argument("--group", default=None, help="device group owner")
 
-    if version >= (2018, 1):
-        devices_add.add_argument(
-            "--health",
-            default=None,
-            choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
-            help="device health",
-        )
-    else:
-        devices_add.add_argument(
-            "--status",
-            default=None,
-            choices=["OFFLINE", "IDLE", "RUNNING", "OFFLINING", "RETIRED", "RESERVED"],
-            help="device status",
-        )
-        devices_add.add_argument(
-            "--health",
-            default=None,
-            choices=["UNKNOWN", "PASS", "FAIL", "LOOPING"],
-            help="device health status",
-        )
+    devices_add.add_argument(
+        "--health",
+        default=None,
+        choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
+        help="device health",
+    )
 
     devices_add.add_argument(
         "--private",
@@ -127,23 +114,58 @@ def configure_parser(parser, version):
     )
 
     # "maintenance"
-    if version >= (2018, 1):
-        devices_maintenance = sub.add_parser(
-            "maintenance", help="maintenance the device"
+    devices_maintenance = sub.add_parser("maintenance", help="maintenance the device")
+    devices_maintenance.add_argument("name", help="device name")
+    devices_maintenance.add_argument(
+        "--force",
+        default=False,
+        action="store_true",
+        help="force device maintenance by canceling running job",
+    )
+    devices_maintenance.add_argument(
+        "--no-wait",
+        dest="wait",
+        default=True,
+        action="store_false",
+        help="do not wait for the device to be idle",
+    )
+
+    # "perms"
+    if version >= (2023, 3):
+        devices_perms = sub.add_parser("perms", help="permissions")
+        perms_sub = devices_perms.add_subparsers(
+            dest="sub_sub_sub_command", help="Sub commands"
         )
-        devices_maintenance.add_argument("name", help="device name")
-        devices_maintenance.add_argument(
-            "--force",
-            default=False,
-            action="store_true",
-            help="force device maintenance by canceling running job",
+        perms_sub.required = True
+
+        perms_add = perms_sub.add_parser("add", help="add permissions")
+        perms_add.add_argument("name", help="name of the device")
+        perms_add.add_argument("group", help="group")
+        perms_add.add_argument("permission", help="permission")
+
+        perms_delete = perms_sub.add_parser("delete", help="delete permissions")
+        perms_delete.add_argument("name", help="name of the device")
+        perms_delete.add_argument("group", help="group")
+        perms_delete.add_argument("permission", help="permission")
+
+        perms_list = perms_sub.add_parser("list", help="list permissions")
+        perms_list.add_argument("name", help="name of the device")
+        out_format = perms_list.add_mutually_exclusive_group()
+        out_format.add_argument(
+            "--json",
+            dest="output_format",
+            default=None,
+            action="store_const",
+            const="json",
+            help="print as json",
         )
-        devices_maintenance.add_argument(
-            "--no-wait",
-            dest="wait",
-            default=True,
-            action="store_false",
-            help="do not wait for the device to be idle",
+        out_format.add_argument(
+            "--yaml",
+            dest="output_format",
+            default=None,
+            action="store_const",
+            const="yaml",
+            help="print as yaml",
         )
 
     # "show"
@@ -212,26 +234,12 @@ def configure_parser(parser, version):
     owner.add_argument("--user", default=None, help="device owner")
     owner.add_argument("--group", default=None, help="device group owner")
 
-    if version >= (2018, 1):
-        devices_update.add_argument(
-            "--health",
-            default=None,
-            choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
-            help="device health",
-        )
-    else:
-        devices_update.add_argument(
-            "--status",
-            default=None,
-            choices=["OFFLINE", "IDLE", "RUNNING", "OFFLINING", "RETIRED", "RESERVED"],
-            help="device status",
-        )
-        devices_update.add_argument(
-            "--health",
-            default=None,
-            choices=["UNKNOWN", "PASS", "FAIL", "LOOPING"],
-            help="device health status",
-        )
+    devices_update.add_argument(
+        "--health",
+        default=None,
+        choices=["GOOD", "UNKNOWN", "LOOPING", "BAD", "MAINTENANCE", "RETIRED"],
+        help="device health",
+    )
 
     display = devices_update.add_mutually_exclusive_group()
     display.add_argument(
@@ -241,35 +249,30 @@ def configure_parser(parser, version):
         "--private", dest="public", action="store_false", help="make the device private"
     )
 
+    if version >= (2022, 4):
+        devices_update.add_argument("--type", default=None, help="device-type")
+
+    if version >= (2025, 12):
+        devices_update.add_argument(
+            "--reason", default=None, help="reason of health change"
+        )
+
 
 def help_string():
     return "manage devices"
 
 
 def handle_add(proxy, options, config):
-    if config["version"] >= (2018, 1):
-        proxy.scheduler.devices.add(
-            options.name,
-            options.type,
-            options.worker,
-            options.user,
-            options.group,
-            not options.private,
-            options.health,
-            options.description,
-        )
-    else:
-        proxy.scheduler.devices.add(
-            options.name,
-            options.type,
-            options.worker,
-            options.user,
-            options.group,
-            not options.private,
-            options.status,
-            options.health,
-            options.description,
-        )
+    proxy.scheduler.devices.add(
+        options.name,
+        options.type,
+        options.worker,
+        options.user,
+        options.group,
+        not options.private,
+        options.health,
+        options.description,
+    )
     return 0
 
 
@@ -281,13 +284,16 @@ def _lookups(value, fields):
             else:
                 value = value[key]
     except IndexError:
-        print("list index out of range (%d vs %d)" % (int(key), len(value)))
+        print(
+            "list index out of range (%d vs %d)" % (int(key), len(value)),
+            file=sys.stderr,
+        )
         return 1
     except KeyError:
-        print("Unknown key '%s' for '%s'" % (key, value))
+        print(f"Unknown key '{key}' for '{value}'", file=sys.stderr)
         return 1
     except TypeError:
-        print("Unable to lookup inside '%s' for '%s'" % (value, key))
+        print(f"Unable to lookup inside '{value}' for '{key}'", file=sys.stderr)
         return 1
     print(value)
     return 0
@@ -310,8 +316,6 @@ def handle_dict(proxy, options, _):
             return _lookups(value, options.field.split("."))
         else:
             # Extract some variables
-            import jinja2
-
             env = jinja2.Environment(autoescape=False)  # nosec
             ast = env.parse(config)
             field_name = options.field.split(".")[0]
@@ -325,14 +329,14 @@ def handle_dict(proxy, options, _):
                         return 0
                     else:
                         return _lookups(value, options.field.split(".")[1:])
-            print("Unknown field '%s'" % field_name)
+            print("Unknown field '%s'" % field_name, file=sys.stderr)
             return 1
 
     else:
         config = options.config.read()
         ret = proxy.scheduler.devices.set_dictionary(options.name, config)
         if not ret:
-            print("Unable to set the configuration")
+            print("Unable to set the configuration", file=sys.stderr)
         return 0 if ret else 1
 
 
@@ -346,21 +350,15 @@ def handle_list(proxy, options, config):
     else:
         print("Devices:")
         for device in devices:
-            if config["version"] >= (2018, 1):
-                print(
-                    "* %s (%s): %s,%s"
-                    % (
-                        device["hostname"],
-                        device["type"],
-                        device["state"],
-                        device["health"],
-                    )
-                )
-            else:
-                print(
-                    "* %s (%s): %s"
-                    % (device["hostname"], device["type"], device["status"])
+            print(
+                "* %s (%s): %s,%s"
+                % (
+                    device["hostname"],
+                    device["type"],
+                    device["state"],
+                    device["health"],
                 )
+            )
     return 0
 
 
@@ -385,6 +383,28 @@ def handle_maintenance(proxy, options, _):
     return 0
 
 
+def handle_perms(proxy, options, _):
+    if options.sub_sub_sub_command == "add":
+        proxy.scheduler.devices.perms_add(
+            options.name, options.group, options.permission
+        )
+    elif options.sub_sub_sub_command == "delete":
+        proxy.scheduler.devices.perms_delete(
+            options.name, options.group, options.permission
+        )
+    elif options.sub_sub_sub_command == "list":
+        perms = proxy.scheduler.devices.perms_list(options.name)
+        if options.output_format == "json":
+            print(json.dumps(perms))
+        elif options.output_format == "yaml":
+            safe_yaml.dump(perms, sys.stdout)
+        else:
+            print("Permissions:")
+            for perm in perms:
+                print("* %s %s" % (perm["group"], perm["name"]))
+    return 0
+
+
 def handle_show(proxy, options, config):
     device = proxy.scheduler.devices.show(options.name)
 
@@ -395,11 +415,8 @@ def handle_show(proxy, options, config):
     else:
         print("name        : %s" % device["hostname"])
         print("device-type : %s" % device["device_type"])
-        if config["version"] >= (2018, 1):
-            print("state       : %s" % device["state"])
-            print("health      : %s" % device["health"])
-        else:
-            print("status      : %s" % device["status"])
+        print("state       : %s" % device["state"])
+        print("health      : %s" % device["health"])
         if config["version"] < (2019, 9):
             print("user        : %s" % device["user"])
             print("group       : %s" % device["group"])
@@ -434,7 +451,7 @@ def handle_tags(proxy, options, _):
 
 
 def handle_update(proxy, options, config):
-    if config["version"] >= (2018, 1):
+    if config["version"] < (2022, 4):
         proxy.scheduler.devices.update(
             options.name,
             options.worker,
@@ -444,6 +461,17 @@ def handle_update(proxy, options, config):
             options.health,
             options.description,
         )
+    elif config["version"] < (2025, 12):
+        proxy.scheduler.devices.update(
+            options.name,
+            options.worker,
+            options.user,
+            options.group,
+            options.public,
+            options.health,
+            options.description,
+            options.type,
+        )
     else:
         proxy.scheduler.devices.update(
             options.name,
@@ -451,9 +479,10 @@ def handle_update(proxy, options, config):
             options.user,
             options.group,
             options.public,
-            options.status,
             options.health,
             options.description,
+            options.type,
+            options.reason,
         )
     return 0
 
@@ -464,6 +493,7 @@ def handle(proxy, options, config):
         "dict": handle_dict,
         "list": handle_list,
         "maintenance": handle_maintenance,
+        "perms": handle_perms,
         "show": handle_show,
         "tags": handle_tags,
         "update": handle_update,
