@@ -20,13 +20,22 @@ import argparse
 import contextlib
 import datetime
 import json
+import os
 import pathlib
 import sys
 import time
 import xmlrpc.client
+from collections import defaultdict
 from urllib.parse import urlparse
 
-from lavacli.utils import exc2str, log_yaml, safe_yaml
+from lavacli.utils import (
+    exc2str,
+    fetch_jobs,
+    fetch_jobs_concurrent,
+    log_yaml,
+    print_error_summary,
+    safe_yaml,
+)
 
 
 def configure_log_options(parser):
@@ -57,6 +66,22 @@ def configure_parser(parser, version):
     jobs_cancel = sub.add_parser("cancel", help="cancel a job")
     jobs_cancel.add_argument("job_id", nargs="+", help="job id")
 
+    if version >= (2023, 10):
+        # "autocancel"
+        jobs_autocancel = sub.add_parser(
+            "autocancel",
+            help="cancel jobs that take longer than job timeout and the specific duration",
+        )
+        jobs_autocancel.add_argument(
+            "--duration", type=int, required=True, help="duration in minutes"
+        )
+        jobs_autocancel.add_argument(
+            "--dry-run",
+            action="store_true",
+            default=False,
+            help="do not cancel the jobs",
+        )
+
     # "config"
     jobs_config = sub.add_parser("config", help="job configuration")
     jobs_config.add_argument("job_id", help="job id")
@@ -68,6 +93,15 @@ def configure_parser(parser, version):
     jobs_definition = sub.add_parser("definition", help="job definition")
     jobs_definition.add_argument("job_id", help="job id")
 
+    # "find-errors"
+    jobs_find_errors = sub.add_parser("find-errors", help="Find errors in recent jobs")
+    jobs_find_errors.add_argument(
+        "--limit", type=int, default=100, help="Number of jobs to fetch"
+    )
+    jobs_find_errors.add_argument(
+        "--since", type=int, default=None, help="Jobs from the last N minutes"
+    )
+
     # "list"
     jobs_list = sub.add_parser("list", help="list jobs")
     jobs_list.add_argument(
@@ -302,6 +336,74 @@ def handle_cancel(proxy, options, _):
     return ret
 
 
+def handle_autocancel(proxy, options, _):
+    # Jobs that still running within its timeout shouldn't be canceled.
+    skip_list = []
+    # A loop is needed as the list api only returns the last 100 jobs.
+    while True:
+        jobs = proxy.scheduler.jobs.list(
+            "RUNNING",
+            None,
+            0,
+            100,
+            None,
+            False,
+            options.duration,
+        )
+
+        if not jobs:
+            break
+
+        job_ids = [job["id"] for job in jobs]
+        if set(job_ids).issubset(skip_list):
+            break
+        jobs = [job for job in jobs if job["id"] not in skip_list]
+
+        duration = datetime.timedelta(minutes=options.duration)
+        for job in jobs:
+            job_id = job["id"]
+            definition = proxy.scheduler.jobs.definition(job_id)
+            data = safe_yaml.load(definition)["timeouts"]["job"]
+            job_timeout = datetime.timedelta(
+                days=data.get("days", 0),
+                hours=data.get("hours", 0),
+                minutes=data.get("minutes", 0),
+                seconds=data.get("seconds", 0),
+            )
+            if duration > job_timeout:
+                try:
+                    if not options.dry_run:
+                        proxy.scheduler.jobs.cancel(job_id)
+                    print(
+                        "[CANCELED] %s: [%s] (%s) - %s"
+                        % (
+                            job_id,
+                            job["submitter"],
+                            job["description"],
+                            job["device_type"],
+                        )
+                    )
+                except xmlrpc.client.Error as exc:
+                    print(
+                        f"Unable to cancel {job_id}: {exc2str(exc, options.uri)}",
+                        file=sys.stderr,
+                    )
+                    return 1
+            else:
+                print(
+                    "[  SKIP  ] %s: [%s] (%s) - %s - timeout not reached yet"
+                    % (
+                        job_id,
+                        job["submitter"],
+                        job["description"],
+                        job["device_type"],
+                    )
+                )
+                skip_list.append(job_id)
+
+    return 0
+
+
 def handle_config(proxy, options, _):
     config = proxy.scheduler.jobs.configuration(options.job_id)
     definition, device, dispatcher, env, env_dut = config
@@ -400,7 +502,7 @@ def handle_list(proxy, options, config):
     return 0
 
 
-if sys.stdout.isatty():
+if os.environ.get("FORCE_COLOR") or sys.stdout.isatty():
     COLORS = {
         "exception": "\033[1;31m",
         "error": "\033[1;31m",
@@ -510,7 +612,7 @@ def handle_logs(proxy, options, config):
             print_logs(
                 [
                     {
-                        "dt": datetime.datetime.utcnow().isoformat(),
+                        "dt": datetime.datetime.now(datetime.timezone.utc).isoformat(),
                         "lvl": "info",
                         "msg": "[lavacli] Failure comment: %s"
                         % details["failure_comment"],
@@ -558,7 +660,7 @@ def handle_resubmit(proxy, options, config):
         print_logs(
             [
                 {
-                    "dt": datetime.datetime.utcnow().isoformat(),
+                    "dt": datetime.datetime.now(datetime.timezone.utc).isoformat(),
                     "lvl": "info",
                     "msg": "[lavacli] Job %s submitted" % job_id,
                 }
@@ -579,7 +681,7 @@ def follow_logs(job_id, proxy, options, config):
             print_logs(
                 [
                     {
-                        "dt": datetime.datetime.utcnow().isoformat(),
+                        "dt": datetime.datetime.now(datetime.timezone.utc).isoformat(),
                         "lvl": "info",
                         "msg": "[lavacli] Seeing %s logs" % job,
                     }
@@ -599,7 +701,7 @@ def handle_run(proxy, options, config):
     print_logs(
         [
             {
-                "dt": datetime.datetime.utcnow().isoformat(),
+                "dt": datetime.datetime.now(datetime.timezone.utc).isoformat(),
                 "lvl": "info",
                 "msg": "[lavacli] Job %s submitted" % job_id,
             }
@@ -615,7 +717,7 @@ def handle_run(proxy, options, config):
             print_logs(
                 [
                     {
-                        "dt": datetime.datetime.utcnow().isoformat(),
+                        "dt": datetime.datetime.now(datetime.timezone.utc).isoformat(),
                         "lvl": "info",
                         "msg": "[lavacli] Seeing %s logs" % job,
                     }
@@ -634,15 +736,17 @@ def handle_run(proxy, options, config):
 def handle_show(proxy, options, config):
     job = proxy.scheduler.jobs.show(options.job_id)
 
+    if options.output_format in ("json", "yaml"):
+        submit_time = job.get("submit_time")
+        job["submit_time"] = submit_time.value if submit_time else None
+        start_time = job.get("start_time")
+        job["start_time"] = start_time.value if start_time else None
+        end_time = job.get("end_time")
+        job["end_time"] = end_time.value if end_time else None
+
     if options.output_format == "json":
-        job["submit_time"] = job["submit_time"].value or None
-        job["start_time"] = job["start_time"].value or None
-        job["end_time"] = job["end_time"].value or None
         print(json.dumps(job))
     elif options.output_format == "yaml":
-        job["submit_time"] = job["submit_time"].value or None
-        job["start_time"] = job["start_time"].value or None
-        job["end_time"] = job["end_time"].value or None
         safe_yaml.dump(job, sys.stdout)
     else:
         print("id          : %s" % job["id"])
@@ -655,6 +759,8 @@ def handle_show(proxy, options, config):
         print("Health      : %s" % job["health"])
         if job.get("failure_comment"):
             print("failure     : %s" % job["failure_comment"])
+        if job.get("error_type"):
+            print("error       : %s: %s" % (job["error_type"], job["error_msg"]))
         print("pipeline    : %s" % job["pipeline"])
         print("tags        : %s" % str(job["tags"]))
         print("visibility  : %s" % job["visibility"])
@@ -727,9 +833,62 @@ def handle_wait(proxy, options, _):
     return 0
 
 
+def handle_find_errors(proxy, options, config):
+    """Main function to fetch and process LAVA jobs."""
+
+    parsed = urlparse(options.uri)
+    host = parsed.netloc.split("@")[-1]
+    url = f"{parsed.scheme}://{host}"
+
+    job_numbers = fetch_jobs(proxy, None, "INCOMPLETE", options.limit, options.since)
+    if not job_numbers:
+        print("No jobs found.")
+        return
+
+    print(f"\nTotal jobs found: {len(job_numbers)}")
+    print("Fetching job details concurrently...")
+
+    error_summary = defaultdict(lambda: defaultdict(list))
+
+    def fetch_job_details(proxy, job_id):
+        """Fetch job details for a single job ID."""
+        return proxy.scheduler.jobs.show(job_id)
+
+    def progress_callback(processed, total):
+        """Show progress every 10 jobs or at the end."""
+        if processed % 10 == 0 or processed == total:
+            print(f"Processed {processed}/{total} jobs...")
+
+    # Fetch all job details concurrently
+    results = fetch_jobs_concurrent(
+        proxy,
+        job_numbers,
+        fetch_job_details,
+        max_workers=20,
+        progress_callback=progress_callback,
+    )
+
+    # Process results
+    for job_id, job_data in results:
+        if isinstance(job_data, Exception):
+            print(f"Error fetching job {job_id}: {job_data}")
+            continue
+
+        if job_data:
+            device_name = job_data.get("device") or job_data.get("device_type")
+            job_url = f"{url}/scheduler/job/{job_id}"
+            error_msg = job_data.get("error_msg", "").strip()
+            if error_msg:
+                error_summary[error_msg][device_name].append(job_url)
+
+    print_error_summary(error_summary, summary=True)
+    print_error_summary(error_summary, summary=False)
+
+
 def handle(proxy, options, config):
     handlers = {
         "cancel": handle_cancel,
+        "autocancel": handle_autocancel,
         "config": handle_config,
         "definition": handle_definition,
         "list": handle_list,
@@ -741,5 +900,6 @@ def handle(proxy, options, config):
         "submit": handle_submit,
         "validate": handle_validate,
         "wait": handle_wait,
+        "find-errors": handle_find_errors,
     }
     return handlers[options.sub_sub_command](proxy, options, config)
