@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # vim: set ts=4
 
 # Copyright 2017 Rémi Duraffort
@@ -18,9 +17,12 @@
 # along with lavacli.  If not, see <http://www.gnu.org/licenses/>
 
 import argparse
-import jinja2
+import sys
 from pathlib import Path
 
+import jinja2
+import ruamel.yaml
+
 from lavacli.utils import safe_yaml
 
 from . import jobs
@@ -77,12 +79,19 @@ def help_string():
 
 
 def handle_logs(_, options):
-    try:
-        logs = safe_yaml.load(options.filename.read())
-    except ruamel.yaml.YAMLError:
-        print("Invalid yaml file")
+    if options.filename.name == "<stdin>":
+        for line in options.filename:
+            try:
+                jobs.print_logs([safe_yaml.load(line)], options.raw, options.filters)
+            except ruamel.yaml.YAMLError:
+                print("Invalid yaml file")
     else:
-        jobs.print_logs(logs, options.raw, options.filters)
+        try:
+            logs = safe_yaml.load(options.filename.read())
+        except ruamel.yaml.YAMLError:
+            print("Invalid yaml file")
+        else:
+            jobs.print_logs(logs, options.raw, options.filters)
 
 
 def handle_templates(_, options):
@@ -107,14 +116,14 @@ def handle_templates(_, options):
     try:
         context = {k.split("=")[0]: "=".join(k.split("=")[1:]) for k in options.context}
     except Exception:
-        print("Invalid context variable")
-        print("Should be of the form key=value")
+        print("Invalid context variable", file=sys.stderr)
+        print("Should be of the form key=value", file=sys.stderr)
         return 1
     try:
         print(template.render(**context))
         return 0
     except jinja2.TemplateNotFound:
-        print("Unable to find the templates")
+        print("Unable to find the templates", file=sys.stderr)
         return 1
 
 
